/***************************************************************************
                          api_base.h  -  description
                             -------------------
    begin                : Thu Jul 31 2003
    copyright            : (C) 2003 by David Rokhvarg
    email                : drokhvarg@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef API_BASE_H
#define API_BASE_H

#include "tester.h"			//all needed Qt header files

#include "TesterDefines.h"  //debug macros

#if defined(BUILD_FOR_UNIX)

  #include <linux/wanpipe.h>	//sdla_exec_t
  #include <linux/sdla_edu.h>	//edu_exec_cmd_t
  #include <linux/sdlapci.h>	//for MAX_SIZEOF_S514_MEMORY
  #include <linux/sdlasfm.h>	// SDLA firmware module definitions

#elif defined(WIN32)

  #include "..\include\sdla\sdla.h"		//for sdla IOCTLs
  #include "..\include\sdlasfm.h"	    /* SDLA firmware module definitions */

#endif

/******** Error codes **********/
#define	SDLAR_SETUP_FAILED				0L
#define	SDLAR_SETUP_OK_SECOND_CALL		1L
#define	SDLAR_SETUP_OK_FIRST_CALL		2L
#define	SDLAR_NOT_REGISTERED			0L
#define	SDLAR_DEREGISTERED				1L
#define	SDLAR_RELEASED					2L
#define	SDLAR_REGISTERED				1L
#define	SDLAR_ALLOCATED					2L
#define	SDLAR_CARD_BLOCKED				0L
#define	SDLAR_PREPARED					2L
#define	SDLAR_NOT_PREPARED				0L
#define	SDLAR_ERROR_PREPARING			3L
#define	SDLAR_ERROR_WAITING_FOR_MUTEX	4L


#define DELAY_TIME			10000
#define MAX_RETRY_COUNT 	1000

/********** Error exit codes ***********/
enum ErrCodes
{
	ERR_SYSTEM = 1,         /* System error */
	ERR_SYNTAX,             /* Command line syntax error */
	ERR_LIMIT
};


/////////////////////////////////////////////////////////////////////
//definitions for "internal_state"
#define CONNECTED_TO_DRIVER BIT0_SET
#define FIRMWARE_LOADED     BIT1_SET

/**All per-protocol API classes will be derived from this class. The class encapsulates functions common for all APIs.
  *@author David Rokhvarg
  */

class api_base {

public:

	api_base();
	~api_base();

#if defined(BUILD_FOR_UNIX)

	/*	Initializes the library, loads firmware.
		Each application instance MUST call this before using any other command.
	
	returns:
		0 						- call was successfull.
		OPEN_DEVICE_ERR			- failed to open wanpipe device.
		MEMORY_ALLOC_ERR		- failed allocate memory for internal needs
		OPEN_FIRMWARE_FILE_ERR	- failed to open firmware file
		IOCTL_ERR				- an IOCTL failed - check /var/log/messages for
								  debugging information.
	*/
	int API_BASE_Setup(char * device_name);

#elif defined(WIN32)

	/*	Each application instance MUST call this on initialization.
	returns:
		SDLAR_SETUP_FAILED -	Failed to load SDLA driver.
								Check SDLA driver is installed and running.
		SDLAR_SETUP_OK_FIRST_CALL - This is the first setup call and the SDLA driver
									loaded successfully.
		SDLAR_SETUP_OK_SECOND_CALL - This is not the first successful setup call
									(driver already loaded).
	*/
	int API_BASE_Setup(void *);	

#endif

	/*
	    Each application instance MUST call this before closing.
		returns 1 always (return code reserved for future use)
	*/
	int API_BASE_Cleanup(void);

	/*
    returns:
		SDLAR_NOT_REGISTERED (0L)	- failed to obtain a selector
			(any read or write calls to the board will result
			in a general protection fault)
		SDLAR_REGISTERED (1L)		- succeeded : the card's
			instance count was incremented but this is not the
			first registration call for this board (it is already

			in use by an application)
		SDLAR_ALLOCATED (2L)		- succeeded : the card's
			instance count was incremented and this is the
			first registration call for this board, and the
			memory selector was allocated
		('cardnumber' is ignored on UNIX-based systems)
	*/
	int API_BASE_RegisterCard(BYTE cardnumber);
	
	/*
    returns:
		SDLAR_NOT_REGISTERED (0L)	- card wasn't registered
		SDLAR_DEREGISTERED (1L)		- card's instance count
			was decremented but isn't currently zero
			(i.e. an application is still using the specified board)
		SDLAR_RELEASED (2L)			- card is now not used by
			any application, and the selector was freed
		('cardnumber' is ignored on UNIX-based systems)
	*/
	int API_BASE_DeregisterCard(BYTE cardnumber);

	/*
    returns a bit record (32-bit) of cards registered for use
	and cards that exist:
	[ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
    For example, if bit R2 is set, then card 2 is registered
	if bit E2 is set, then card 2 exists
 			(i.e. a configuration entry exists
			in SDLA.INI)
	*/
	int API_BASE_GetCards(void);
	
	/* Undocumented exports - use at your own risk */

	/*	Call API_BASE_PrepareForLoading() before loading the card.
		If card is busy do not call
		API_BASE_LoadFirmware() - somebody is using the card.
		If not busy, the card will be marked as "Loading in progress"
		until API_BASE_LoadComplete() is called.
		('cardnumber' is ignored on UNIX-based systems)
	*/
	int API_BASE_PrepareForLoading(BYTE cardnumber);

	/*	Load the firmware and start it. No other calls permitted
		while loading is in progress.
		('cardnumber' is ignored on UNIX-based systems)
	*/
	int API_BASE_LoadFirmware(char * firmware_file, BYTE cardnumber);

	//	Card will no longer be marked as "Loading in progress".
	int API_BASE_LoadComplete(BYTE cardnumber);
	
protected:

	/** prototypes of functions which SHOULD NOT be used directly by an application **/
	unsigned char API_BASE_ReadBlock(ULONG offset, ULONG length, void * destination_ptr);
	void API_BASE_WriteBlock(ULONG offset, ULONG length, void * source_ptr);

	char SetWaitOppFlag(WORD offset);

#if defined(BUILD_FOR_UNIX)

  int my_sleep(int usecs);
  int DoIoctl();

#elif defined(WIN32)

  int my_sleep(int milliseconds);

	/* set default timeout to 2 seconds */
	DWORD	timeout_ticks;
	HANDLE	api_base_control_mutex, cardoperation_mutex;

#endif

	BYTE	cardnumber;
	unsigned char internal_state;

	char * progname;
	char * err_messages[2];

private:

	//zero adapter's memory. used before downloading the firmware.
	void API_BASE_ZeroWindow();

	void API_BASE_StartS514(BYTE cardnumber);
	void API_BASE_StopS514(BYTE cardnumber);

	unsigned char * data;
	unsigned char * data_bakup;

#if defined(BUILD_FOR_UNIX)

	/********** Strings & message tables.****************/
	int dev;
	sdla_exec_t * exec;
	edu_exec_cmd_t * ctrl;

#elif defined(WIN32)

	DWORD	ln;		// dummy variable to hold returned bytes from DevIOCnt
	HANDLE	frdrv;

#endif

};

#endif
