/*****************************************************************************
* fapi.cpp	Interface/API provider to Sangoma Frame Relay code.
*
* Author: 	David Rokhvarg <drokhvarg@sangoma.com>
*		
*
* Copyright:	(c) 2001 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
* Sep 18, 2001  David Rokhvarg  Initial version.
*  				This version running above new microcode vT.20
*  				which is using separate mailbox for sending data.
* 			
******************************************************************************/

#include "fapi.h"	//class header file

#define DBG_FAPI 0  //if 0 no debug output, else there will be debug output

#ifndef min
#define min(a,b)        (((a)<(b))?(a):(b))
#endif

/********************* Functions Implementation **************************/

Fapi::Fapi()
{
	// line tracing specific data and functions
	num_frames = 0;
	curr_trace_addr = 0;
	start_trace_addr = 0;
	available_buffer_space = 0;
}

Fapi::~Fapi()
{
	DbgPrint1(DBG_FAPI, "~Fapi()\n");
}

int Fapi::FAPIAlive(void)
{
	return(1);
}

/*******************************************************************
* Show error message.
*******************************************************************/
void Fapi::show_error (int err)
{
	if (err == ERR_SYSTEM) fprintf(stderr, "%s: SYSTEM ERROR %d: %s!\n",
		progname, errno, strerror(errno))
		;
	else fprintf(stderr, "%s: %s!\n", progname,
		err_messages[min(err, ERR_LIMIT) - 2])
		;
}

char Fapi::FAPIControl(FCBLOCK * block)
{
	char rcode;
	
#if defined(BUILD_FOR_UNIX)
	
#elif defined(WIN32)
	DWORD mresult;
	
	// start system-wide critical section
	// release mutex in SayReturnResult!!!
	mresult = WaitForSingleObject(api_base_control_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED) {
		block->return_code = Z80_TIMEOUT_ERROR;
		return block->return_code;
	}
#endif
	
	if(!(internal_state & CONNECTED_TO_DRIVER) || !(internal_state & FIRMWARE_LOADED))
	{
		DbgPrint1(DBG_FAPI, "Fapi::FAPIControl() : not connected!!!\n");
		rcode = block->return_code = Z80_TIMEOUT_ERROR;
		
		// someone else can do a mail box call now
#if defined(BUILD_FOR_UNIX)
		
#elif defined(WIN32)
		ReleaseMutex(api_base_control_mutex);
#endif
		
		return rcode;
	}
	
	rcode = InternalDoControl(block);
	
	// someone else can do a mail box call now
#if defined(BUILD_FOR_UNIX)
	
#elif defined(WIN32)
    ReleaseMutex(api_base_control_mutex);
#endif
	
	return(rcode);
}

char Fapi::InternalDoControl(FCBLOCK * block)
{
	//char s[80];
	char flag;
	//DWORD mresult;
	ULONG offset, num_bytes;
	void * buffer;
	
	DbgPrint2(DBG_FAPI, "InternalDoControl():block->command		    0x%x.\n", block->command);
	DbgPrint2(DBG_FAPI, "InternalDoControl():block->buffer_length	0x%x.\n", block->buffer_length);
	DbgPrint2(DBG_FAPI, "InternalDoControl():block->dlci			    0x%x.\n", block->dlci );
	
	offset = FMO_SENDBOX+1;
	
	//send is using separate mailbox on v2.0
	if(block->command == FR_INFORMATION_WRITE)
		offset += 0x800L;
	
	// reject block if buffer_length is too high, no matter what
	if (block->buffer_length > FR_MDATALEN)
		return(SayReturnResult(	block->return_code=DATA_LENGTH_TOO_BIG,
								block->data[0], (block->buffer_length > 0)));
	
	switch(block->command)
	{
	case FR_INFORMATION_WRITE:
		{
			int ind;
			
			//DbgPrint2(DBG_FAPI, "InternalDoControl(): block->buffer_length = %d\n",
			//				block->buffer_length);
			sdla_card.temp_tx_buffer_length = block->buffer_length;
			
			for(ind=0; ind < block->buffer_length;ind++)
			{
				//DbgPrint2(DBG_FAPI, "InternalDoControl():block->data[%d] = 0x%x\n",
				//ind, block->data[ind] );
				sdla_card.temp_tx_data_buff[ind] = block->data[ind];
			}
		}
		
		break;
		
	case FR_INFORMATION_READ:
		if (!ReadData(block))
		{	return(SayReturnResult(block->return_code=NO_DATA_AVAILABLE,
		block->data[0], block->buffer_length));
		}else
		{	return(SayReturnResult(block->return_code, block->data[0],
		block->buffer_length));
		}
		break;
		
	case FR_GET_TRACE_INFO:
		block->buffer_length = GetTraceInfo(block);
		return(SayReturnResult(block->return_code = 0, block->data[0],
				       	block->buffer_length));
		break;
		
	}
	
	// submit to appropriate mailbox via driver
	num_bytes = block->buffer_length + FR_CB_HEADERLEN;
	buffer = block;
	
	//copy control block onto the card
	API_BASE_WriteBlock(offset, num_bytes, (void *)block);
	
	// submitted to mailbox... now set flag and wait to clear
	flag = SetWaitOppFlag((unsigned short)offset-1);
	
	if (!flag){
		block->return_code = Z80_TIMEOUT_ERROR;
	}else
	{	// give application a copy of the mailbox after the transaction
		num_bytes = FR_CB_HEADERLEN;
		
		API_BASE_ReadBlock(offset, num_bytes, block);
		
		//if there is data on return from command copy it block->data.
		num_bytes = block->buffer_length;
		buffer = block->data;
		offset += FR_CB_HEADERLEN;
		
		API_BASE_ReadBlock(offset, num_bytes, block->data);	       	
	}
	
	if(block->command == FR_SET_DLCI_CONFIGURATION && block->return_code == 0)
		RetrieveBufferPointers();
	
	if(block->command == FR_INFORMATION_WRITE && block->return_code == 0)
		SendFrame(block);
	
	if( block->command == FR_SET_TRACE_CONFIGURATION &&
		block->return_code == 0 &&
		block->data[0] == 0x37)
	{	//it is Enable Tracing request.
		SetTraceParam(block);
		
	}else if(block->command == FR_SET_TRACE_CONFIGURATION &&
		block->return_code == 0 &&
		block->data[0] != 0x37)
	{	//it is Disable Tracing request.
		//do nothing
		SetTraceParam(block);
	}
	
	DbgPrint2(DBG_FAPI, "InternalDoControl():block->return_code	0x%x.\n", block->return_code );
	
	return(SayReturnResult(block->return_code, block->data[0], block->buffer_length));
}

// 's' must be of size less than (80-5) bytes incl. null
char Fapi::SayReturnResult(char rval, char dval, WORD bl)
{
	char a[80];
	if (bl > 0)
		sprintf(a, "0x%02x  Data[0] = 0x%02x", (BYTE)rval, (BYTE)dval);
	else
		sprintf(a, "0x%02x", (BYTE)rval);
	
	//DbgPrint2(DBG_FAPI, "SayReturnResult : %s\n", a);
	
	return(rval);
}

unsigned char Fapi::ReadData(FCBLOCK * block)
{
	UCHAR rxel[16];
	USHORT rawlength;
	ULONG ptr_data_bfr;
	UCHAR tmp = 0;
	ULONG offset, num_bytes;
	void * buffer;
	
	DbgPrint1(DBG_FAPI, "ReadData()\n");
	
	buffer = rxel;
	offset = sdla_card.rx_curr514;
	num_bytes = 16;
	
	// copy receive element
	API_BASE_ReadBlock(offset, num_bytes, buffer);
	
	//
	// test if opp_flag is set
	//
	if (rxel[0] == 1)
	{	ptr_data_bfr = *(ULONG *)&rxel[0x0c];
		rawlength = *(USHORT *)&rxel[1];
		
		DbgPrint2(DBG_FAPI, "ReadData(): rawlength	: %d\n", rawlength);
		DbgPrint2(DBG_FAPI, "ReadData(): ptr_data_bfr: 0x%x\n", ptr_data_bfr);
		
		// copy the entire receive element to user buffer
		memcpy(block->data, rxel, 16);
		
		// now copy actual packet data
		buffer = (UCHAR *)&block->data[16];
		offset = ptr_data_bfr;
		num_bytes = rawlength;
		
		
		API_BASE_ReadBlock(offset, num_bytes, buffer);
		
		// reset opp_flag
		//API_BASE_WriteByte(sdla_card.rx_curr514, 0);
		API_BASE_WriteBlock(sdla_card.rx_curr514, 1, &tmp);
		
		//move to the next rx element
		sdla_card.rx_curr514 += 0x10;
		if (sdla_card.rx_curr514 == (sdla_card.rx_end514 + 0x10))
			sdla_card.rx_curr514 = sdla_card.rx_base514;
		
		block->return_code = 0;
		block->buffer_length = rawlength;
		
		return 1;
	}else
	{	DbgPrint1(DBG_FAPI, "ReadData():Nothing to read.\n");
		return 0;
	}
}

void Fapi::RetrieveBufferPointers()
{
	USHORT usv;
	UCHAR tbfr[20];
	ULONG offset, num_bytes;
	void * buffer;
	
	DbgPrint1(DBG_FAPI, "RetrieveBufferPointers()\n");
	
	buffer = tbfr;
	offset = RX_BFR_CFG_PTR;
	num_bytes = 20;
	
	// copy receive bfr config stuff
	API_BASE_ReadBlock(offset, num_bytes, buffer);
	
	// get number of rx buffers
	usv = *(USHORT *)&tbfr[0];
	sdla_card.rx_base514 = *(ULONG *)&tbfr[2];
	sdla_card.rx_curr514 = *(ULONG *)&tbfr[6];
	sdla_card.rx_bfr_base514 = *(ULONG *)&tbfr[10];
	sdla_card.rx_bfr_end514 = *(ULONG *)&tbfr[16];
	
	//length of Receive Status Elements array
	sdla_card.rx_end514 = sdla_card.rx_base514 + (usv -1)*16;
	DbgPrint1(DBG_FAPI, "Read pointer information (Rx):\n");
	DbgPrint2(DBG_FAPI, "number_Rx_status_elements = %d\n", usv);
	DbgPrint3(DBG_FAPI, "rx_base514 = 0x%08X, rx_end514 = 0x%08X\n",
		sdla_card.rx_base514, sdla_card.rx_end514);
}


/////////////////////////////////////////////////////////////////////////////////////////

UCHAR Fapi::SendFrame(FCBLOCK* block)
{
	ULONG lp, data_ptr;
	ULONG offset, num_bytes;
	void * buffer;
	UCHAR tmp = 1;
#define DATA_PTR_OFFSET	0x0C
	
	DbgPrint1(DBG_FAPI, "SendFrame()\n");
	
	buffer = &lp;
	offset = MAILBOX_ADDRESS+MAILBOX_DATA+0x800L;
	num_bytes = 4;//it is ULONG
	
	//
	// INFORMATION_WRITE returned zero, there is address of next tx element at data area.
	// get Tx element
	//
	API_BASE_ReadBlock(offset, num_bytes, buffer);
	
	DbgPrint2(DBG_FAPI, "SendFrame():send using element 0x%x.\n", lp);
	DbgPrint1(DBG_FAPI, "SendFrame():Getting data pointer\n");
	
	buffer = &data_ptr;
	offset = lp+DATA_PTR_OFFSET;
	num_bytes = 4;//it is ULONG
	
	// get data pointer - where to put tx data
	API_BASE_ReadBlock(offset, num_bytes, buffer);
	
	DbgPrint2(DBG_FAPI, "SendFrame():data pointer 0x%x.\n", data_ptr);
	
	//
	//Transfer data to the s514
	//
	//using data and buffer length saved in the global array
	buffer = &sdla_card.temp_tx_data_buff[0];
	offset = data_ptr;
	num_bytes = sdla_card.temp_tx_buffer_length;
	
	API_BASE_WriteBlock(offset, num_bytes, buffer);
	
	//
	// set opp_flag to start transmitting
	//
	//API_BASE_WriteByte(lp, 1);
	
	API_BASE_WriteBlock(lp, 1, &tmp);
	
	DbgPrint1(DBG_FAPI, "SendFrame():finished SendFrame\n");
	return 0;
}


/////////////////////////////////////////////////////////////////////////////////////
// The fucntion sets up the trace parameters.
// It's to be called before any GET_TRACE_INFO executed.
void Fapi::SetTraceParam(FCBLOCK * block)
{
	ULONG offset, num_bytes;
	
	DbgPrint1(DBG_FAPI, "SetTraceParam()\n");
	
	offset = ADDR_NO_TRC_ELEMENTS;
	num_bytes = 2;//it is USHORT
	// get number of trace status elements
	API_BASE_ReadBlock(offset, num_bytes, &num_frames);
	
	DbgPrint2(DBG_FAPI, "SetTraceParam():number of trace status elements : %d\n", num_frames);
	
	// get address of first status element
	offset = ADDR_NO_TRC_ELEMENTS + 0x02;
	num_bytes = 4;//it is ULONG
	API_BASE_ReadBlock(offset, num_bytes, &curr_trace_addr);
	
	DbgPrint2(DBG_FAPI, "SetTraceParam():address of first status element : 0x%x\n",
		curr_trace_addr);
	
	// store the begining to later use
	start_trace_addr = curr_trace_addr;
	//calculate available buffer space ( can be moved from here, actually )
	available_buffer_space = 1926;
	return;
}

////////////////////////////////////////////////////////////////////////////////////

USHORT Fapi::GetTraceInfo(FCBLOCK * block)
{
	ULONG		frames = 0;
	PFRAME_DATA fr_data;
	ULONG		data_ptr;
	ULONG		loops;
	USHORT		buffer_length = 0;
	UCHAR		tmp = 1;
	ULONG		offset = 0, num_bytes = 0;
	void *		buffer;
	
	DbgPrint1(DBG_FAPI, "GetTraceInfo()\n");
	
	block->data[0] = 0;
	
	// the user buffer has no more space than for 32 frames on one hand. 
	// On the other hand we don't want to do more that one circle, in order
	// not to get to some race condition
	loops = (num_frames < 32) ? num_frames : 32;
	
	for(frames=0; frames < loops; frames++)
	{
		DbgPrint2(DBG_FAPI, "GetTraceInfo() : curr_trace_addr : 0x%X\n",
			curr_trace_addr
			);

		DbgPrint2(DBG_FAPI, "cardnumber : %d\n", cardnumber);

		// check the opp_flag if we get something there... if not - exit
		if(API_BASE_ReadBlock(curr_trace_addr, 1, &tmp) == 0x00)
			break;
		
		// v1.3 check if we have a space for the FRAME_INFO structure
		if ( available_buffer_space < (1+sizeof(FRAME_DATA) + buffer_length) )
		{	// indicate that we have more frames on board and exit
			block->data[0] |= MORE_TRC_DATA;
			break;
		}
		
		// prepare pointer to frame info
		fr_data = (PFRAME_DATA)(&block->data[buffer_length]);
		
		// start to fill in ...
		fr_data->status = API_BASE_ReadBlock(curr_trace_addr + 0x05, 1, &tmp);
		
		buffer = &fr_data->time_stamp;
		offset = curr_trace_addr + 0x06;
		num_bytes = 2;//it is USHORT value there
		
		API_BASE_ReadBlock(offset, num_bytes, buffer);
		
		// get the frame length
		buffer = &fr_data->real_length;
		offset = curr_trace_addr + 0x01;
		num_bytes = 2;//it is USHORT
		
		API_BASE_ReadBlock(offset, num_bytes, buffer);
		
		// get ptr to actual data
		buffer = &data_ptr;
		offset = curr_trace_addr + 0x0C;
		num_bytes = 4;//it is ULONG
		
		API_BASE_ReadBlock(offset, num_bytes, buffer);
		
		//
		// v1.3 we have to figure out if we can fit the frame into the user buffer.
		//
		// First of all if there was no space on board for the frame or if the frame
		// cannot be fit even into the empty user buffer - we don't wanna fetch it at all
		if ( (data_ptr == 0) ||
			(( fr_data->real_length+sizeof(FRAME_DATA) ) > available_buffer_space) )
		{
			fr_data->passed_length = 0;
		}
		else
		{
			if(available_buffer_space <	fr_data->real_length+sizeof(FRAME_DATA) + buffer_length)
			{	// we'll be able to take it next time
				block->data[0] |= MORE_TRC_DATA;
				break;
			}
			// everything is ok - we can get the frame
			fr_data->passed_length = 1;
			// get the data
			buffer = &fr_data->data[0];
			offset = data_ptr;
			num_bytes = fr_data->real_length;
			
			API_BASE_ReadBlock(offset, num_bytes, buffer);
		}
		
		// zero the opp_flag to show that we got the frame
		tmp = 0;
		API_BASE_WriteBlock(curr_trace_addr, 1, &tmp);
		
		// now when we are done with this frame - move to another
		curr_trace_addr += 16;
		// check if we passed the last address - if so - move to begining
		if (curr_trace_addr == (start_trace_addr + num_frames*16))
			curr_trace_addr = start_trace_addr;
		
		// update the buffer_length. We have to make sure here that we're still on even
		// boundary. If not so - pad a byte
		buffer_length += sizeof(FRAME_DATA);
		if (fr_data->passed_length) buffer_length += fr_data->real_length - 1;
		if (buffer_length & 0x0001) buffer_length++;
	}
	
	// Ok - we get all the stuff inside the user buffer. Now just set the first 5 bits of it
	// to indicate the total number of frames passed
	block->data[0] |= ((frames << 1) & (MAX_FRMS_TRACED << 1));
	
	return(buffer_length);
}

////////////////////////////////////////////////////////////////////////////////////

