/* frelay.h - memory locations and bit masks for HDLC */

/* mailbox usage:
OFFSET	SIZE	CONTENT
0	1	Set to 1 after rest of mailbox set to perform command and
wait for a reset to 0 to indicate command completion
1	1	command code
2	2	buffer length (size in bytes of data field following mailbox)
4	1	return code
5	2	DLCI
7       1       FECN, BECN DE & C/R bits
8	2	num recv frames discarded at interrupt lvl
10	4	num recv frames discarded at app lvl
14	2	reserved
16	var.	data field (max 4096)

  Mailbox is updated accordingly when command is completed, such as
  the return code.
*/

#ifndef FRELAY_H
#define FRELAY_H


#define FMO_SENDBOX		0xE000  // send mailbox
#define FMO_RECVBOX     0xEf60  // receive mailbox
#define FMO_TSTAMP      0		// store current time here UNUSED
#define FMO_FTSTAT      0x1ff1  // tx status bits
#define	FMO_FRSTAT		0x1ff2	// receive status bits
#define	FMO_EVENTS		0x1ff3
#define	FMO_MODEMSTAT	0x1ff4

/* modem bitmasks */
#define	FBM_DCD			0x08
#define	FBM_CTS			0x20


//////////////////////////////////////////////////////////////////////
//
// ===== MAILBOX COMMANDS =====
// For use with IoctlCodeCblockCommand.
//
#define FR_INFORMATION_WRITE        0x01	// not used directly
#define	FR_INFORMATION_READ		    0x02
#define	FR_ISSUE_SIGNALLING_FRAME	0x03
#define FR_SET_DLCI_CONFIGURATION   0x10
#define	FR_READ_DLCI_CONFIGURATION	0x11
#define FR_DISABLE_COMMUNICATIONS   0x12	// also terminates IoctlCodeReadCommand if pending
#define FR_ENABLE_COMMUNICATIONS    0x13
#define FR_READ_DLC_STATUS          0x14
#define FR_READ_DLC_STATISTICS      0x15
#define	FR_FLUSH_DLC_STATISTICS	    0x16
#define	FR_LIST_ACTIVE_DLCI		    0x17
#define	FR_FLUSH_INFO_BUFFERS		0x18	// not to be used
#define FR_ADD_DLCI				    0x20
#define	FR_DELETE_DLCI				0x21
#define FR_ACTIVATE_DLCI			0x22
#define	FR_DEACTIVATE_DLCI			0x23
#define	FR_READ_MODEM_STATUS		0x30
#define	FR_SET_MODEM_STATUS		    0x31
#define	FR_READ_COMMS_ERR_STATS	    0x32
#define FR_FLUSH_COMMS_ERR_STATS    0x33
#define FR_READ_CODE_VERSION        0x40
#define FR_DISCARD_FRAMES           0x41
#define FR_SET_INTERRUPT_TRIGGERS   0x50	// not to be used
#define FR_READ_INTERRUPT_TRIGGERS  0x51

#define FR_SET_TRACE_CONFIGURATION	0x60
#define FR_READ_TRACE_STATISTICS	0x61
#define FR_GET_TRACE_INFO			0x64

/////////////////////////////////////////////////////////////
//Return Codes from mail box commands
//

#define FRRES_OK			0x00	/* command executed successfully */

#define	FRRES_DISABLED		0x01	/* communications not enabled */
#define	FRRES_INOPERATIVE	0x02	/* channel inoperative */
#define	FRRES_DLCI_INACTIVE	0x03	/* DLCI is inactive */
#define	FRRES_DLCI_INVALID	0x04	/* DLCI is not configured */
#define	FRRES_TOO_LONG		0x05
#define	FRRES_TOO_MANY		0x06
#define	FRRES_CIR_OVERFLOW	0x07	/* Tx throughput has exceeded CIR */
#define	FRRES_BUFFER_OVERFLOW	0x08


#define	FRRES_MODEM_FAILURE	0x10	/* DCD and/or CTS dropped */
#define	FRRES_CHANNEL_DOWN	0x11	/* channel became inoperative */
#define	FRRES_CHANNEL_UP	0x12	/* channel became operative */
#define	FRRES_DLCI_CHANGE	0x13	/* DLCI status (or number) changed */
#define	FRRES_DLCI_MISMATCH	0x14
#define	FRRES_INVALID_CMD	0x1F	/* invalid command */


//////////////////////////////////////////////////////////////
//
// Driver-defined return codes for CBLOCK-associated commands.
//
#define RCODE_TIMEOUT_ERROR		0x7f	// Card is not responding to command.
#define RCODE_INIT_ERROR		0x7e	// Card could not be initialized.
// (SET_DLCI_CONFIGURATION only)

//
// All structures below must be packed on 1-byte boundaries.
//

#pragma pack(1)

//
// Control block structure used for IoctlCodeCblockCommand and
// IoctlCodeWriteCommand, as input and output data.
//

typedef struct _FRAME_RELAY_CBLOCK {
	UCHAR OpFlag;			// internal use only
	UCHAR Command;			// frame relay command code
	USHORT BufferLength;	// length of data associated with command
	UCHAR ReturnCode;		// return code
	USHORT DLCI;			// DLCI
	UCHAR FecnBecnDeCr;		// various frame-relevant bits
	UCHAR reserved[8];
	UCHAR Data[1];			// variable-length array containing data
} FRAME_RELAY_CBLOCK, *PFRAME_RELAY_CBLOCK;


//
// May be used as an aid for filling in the data area of the
// SET_DLCI_CONFIGURATION (DLCI=0) command. See printed manual.
//
typedef struct _FRAME_RELAY_CONFIG {
	USHORT Station;
	USHORT ConfigBits;
	USHORT BaudRateKbps;
	USHORT ClockingInterface;
	USHORT MaxIFrameLength;
	USHORT T391;
	USHORT T392;
	USHORT N391;
	USHORT N392;
	USHORT N393;
	USHORT CIRfwd;
	USHORT Bcfwd;
	USHORT Befwd;
	USHORT CIRbwd;
	USHORT Bcbwd;
	USHORT Bebwd;
	USHORT DLCI[1];
} FRAME_RELAY_CONFIG, *PFRAME_RELAY_CONFIG;

//
// Information returned by IoctlCodeStatusCommand.
// See printed manual section on interface area bytes.
//
typedef struct _FRAME_RELAY_STATUS_INFO {
	UCHAR BasePortRegister;			// contents of base port register
	UCHAR ExceptionConditionByte;	// contents of exception condition byte
	UCHAR ModemStatusByte;			// contents of modem status byte
} FRAME_RELAY_STATUS_INFO, *PFRAME_RELAY_STATUS_INFO;

//
// Equivalent to Receive Status Element in Manual.
// Data of variable length follows.
//
typedef struct _FRAME_RELAY_READ_INFO {
	UCHAR Status;					// set to 0x01 on return if frame read
	USHORT FrameLength;				// set to length of frame that follows
	USHORT DLCI;					// DLCI that data arrived on
	UCHAR FecnBecnDeCr;				// various frame-related bits
	UCHAR Reserved[10];
	UCHAR Data[1];					// variable-length frame data
} FRAME_RELAY_READ_INFO, *PFRAME_RELAY_READ_INFO;

//this is used by configurator
typedef struct{
	USHORT CardType;
	
	ULONG  BusNumber;
	ULONG  SlotNumber;
	USHORT CPU;
	USHORT comms_port;
	
	ULONG  Memory;
	ULONG  IRQ;
	ULONG  IO_Port;
	
}SANGFRAPI_DEVICE;

#pragma pack()

//////////////////////////////////////////////////////////////////////

#define MAILBOX_ADDRESS		0xE000
#define MAILBOX_DATA		16
#define MAILBOX_RCODE		4
#define MAILBOX_BUFFER_LENGTH	2
#define MAILBOX_COMMAND		1
#define MAILBOX_DLCI		5
#define MAILBOX_MAX_DATA	1024		/* max. data length in the mailbox */

#define BASE_INTERFACE_ADDRESS  0xE000

#define TIMEOUT_INTERVAL	200
#define INITIALIZE_TIMEOUT_ERROR	0x7e

/* SDLA adapter port constants */
#define OFF_PC_ADDR_WS_SELECT 	0x01	/* the offset from the card I/O port base address of the adapter */
/* segment and PC window select latch */
#define OFF_Z80_WIN_SELECT 	0x02	/* the offset from the card I/O port base address of the Z80 window */
/* select latch */
/* card 64K segment address settings */
/* bits 4-5 of configuration byte written to port (base address + 1) */
#define SEG_D                   0x08
#define SEG_A                   0x00
#define SEG_C                   0x10
#define SEG_E                   0x18

/*===== STATUS BITS ON BOARD =====*/
#define INTERRUPT_INTERFACE_BYTE        0x01010
#define INTERRUPT_INTERFACE_BYTE_514    0xF010

#define INTERRUPT_PERMISSION_BYTE       0x01011
#define INTERRUPT_PERMISSION_BYTE_514   0xF011

#define RECEIVE_INT_BIT                 0x01
#define RECEIVE_INT_MASK                0xfe
#define STANDARD_WINDOW         0x07
#define EXCEPTION_BYTE			0x1003
#define MODEM_INTERFACE_BYTE    0x1004

#define RX_BFR_CFG_PTR          0xf120

#define TX_PTR_TRANSMIT_INT		0x1012

#define CODE_VER_WIN_OFFSET     0x1918
#define CODE_VERSION_WINDOW     0x01


//////////////////////////////////////////
#define	UNUSED_CMD_FOR_EVENTS		0x7f

#define	FR_DATA_LENGTH_TOO_BIG	0x05
#define	NO_BUFFS		0x08
#define	FR_NO_DATA_AVAILABLE	0x05
/////////////////////////////////////////

#pragma pack(1)
typedef struct _MAILBOX {
    UCHAR opp_flag;
    UCHAR command;
    USHORT buffer_length;
    UCHAR rcode;
    USHORT dlci;
    UCHAR fecn_becn_de_cr_bits;
    UCHAR reserved[8];
    UCHAR data[1];
} MAILBOX;
#pragma pack(4)

typedef struct _BUFFER_INFO {
	USHORT num_tx_buffers;
	USHORT num_rx_buffers;
	USHORT rx_index_offset;
	USHORT current_rx_index;
	USHORT current_tx_index;
} BUFFER_INFO;

typedef struct {
	USHORT offset;
	UCHAR window;
} woffptr;

//
//Trace related definitions
//
//
// All structures below must be packed on 1-byte boundaries.
//

#pragma pack(1)
//
// Structures for tracing
//
// The one frame trace;
typedef struct {
		UCHAR	status;			// equals to 0x00 if the frame is incoming, 0x01 if outgoing
								// for the first frame of the bunch of frames sent to user the 
								// bits 3-7 of this byte tells number of frames passed, and bit 1 tell
								// if there are more frames already available on the board
								//
		UCHAR	passed_length;	// equals to 0 if no data passed. Otherwise 1, which means that real_length
								// bytes passed.
		USHORT  real_length;	// real(original) frame length
		USHORT  time_stamp;		// the frame's time stamp
		UCHAR	data[1];		// first byte of the frame data
} FRAME_DATA, *PFRAME_DATA;

#pragma pack()

////////////////////////////////////////////////////////////////////
/* tracing management commands (does not go directly to the card) */
#define FR_ENABLE_TRACING				0x62
#define FR_DISABLE_TRACING			0x63
#define FR_GET_TRACE_INFO				0x64

#define TRACE_WINDOW				    0x04
#define TRACE_CONF_LIST_OFFSET	0x1000
#define ADDR_NO_TRC_ELEMENTS		0x9000

#define MORE_TRC_DATA		0x80
#define MAX_FRMS_TRACED		0x07

/* for FT1 monitoring */
#define FR_FT1_MONITOR_STATUS_CTRL			0x80		/* set the status of the FT1 monitoring */
#define FR_SET_FT1_MODE						0x81		/* set the operational mode of the FT1 module */

#define	UNUSED_CMD_FOR_EVENTS				0x7f

/* return codes */
#define	DATA_LENGTH_TOO_BIG					0x05
#define	NO_BUFFS							0x08
#define	NO_DATA_AVAILABLE					0x05
#define S508_FT1_ADPTR_NOT_PRESENT			0x01
#define S508_FT1_MODE_SELECTION_BUSY		0x02

#define MORE_TRC_DATA		0x80
#define MAX_FRMS_TRACED		0x07

// Macro GET_USHORT_FROM_UCHAR_ARRAY( to, from_arr, from_pos )
// The macro should be used if you want to read some 2 byte piece of UCHAR array
// as a USHORT value.
//
// Example of usage:
// 
// Instead of writing 
//		to = *(USHORT*)&from_arr[from_pos];
// write
//		GET_USHORT_FROM_UCHAR_ARRAY( to, from_arr, from_pos );
//
#define GET_USHORT_FROM_UCHAR_ARRAY( to, from_arr, from_pos )	\
		{																                          \
			PUCHAR to_ptr = (PUCHAR)&(to);										      \
			to_ptr[0] = from_arr[(from_pos)];							          \
			to_ptr[1] = from_arr[(from_pos)+1];							        \
    }

// Macro GET_ULONG_FROM_UCHAR_ARRAY( to, from_arr, from_pos )
// The macro should be used if you want to read some 4 byte piece of UCHAR array
// as a ULONG value.
//
// Example of usage:
// 
// Instead of writing 
//		to = *(ULONG*)&from_arr[from_pos];
// write
//		GET_ULONG_FROM_UCHAR_ARRAY( to, from_arr, from_pos );
//
#define GET_ULONG_FROM_UCHAR_ARRAY( to, from_arr, from_pos )	\
		{																                          \
			ULONG qwer_abcde;											                  \
			PUCHAR to_ptr = (PUCHAR)&(to);										      \
			for(qwer_abcde=0; qwer_abcde<4; qwer_abcde++)				    \
				to_ptr[qwer_abcde] = from_arr[(from_pos)+qwer_abcde];	\
		}


#endif //FRELAY_H
